unit NBStream;

interface

uses NLib;

type
  TBStream = class(TStream)
  private
    Buffer: PChar;
    BufferRemain: Integer;
    BufferSize: Integer;
    Stream: TStream;
  public
    constructor Create(const Stream: TStream; BufferSize: Integer=8192);
        virtual;
    destructor Destroy; override;
    function Seek(Offset: Int64; Whence: TSeekWhence=swSet): Int64;
        override;
    function Seekable: Boolean; override;
    function Size: Cardinal; override;
    procedure Flush; virtual; abstract;
  end;

  TBInStream = class(TBStream)
  private
    BufferCurrent: Integer;
  public
    constructor Create(const Stream: TStream; BufferSize: Integer=8192);
        override;
    function Read(var Buf; NBytes: Cardinal): Cardinal; override;
    function Write(const Buf; NBytes: Cardinal): Cardinal; override;
    procedure Flush; override;
  end;

  TBOutStream = class(TBStream)
  public
    constructor Create(const Stream: TStream; BufferSize: Integer=8192);
        override;
    function Read(var Buf; NBytes: Cardinal): Cardinal; override;
    function Write(const Buf; NBytes: Cardinal): Cardinal; override;
    procedure Flush; override;
  end;

implementation

constructor TBStream.Create(const Stream: TStream; BufferSize: Integer);
begin
  Self.Stream := Stream;
  Self.BufferSize := BufferSize;
  GetMem(Buffer, Self.BufferSize);
end;

destructor TBStream.Destroy;
begin
  Flush;
  FreeMem(Buffer);
end;

function TBStream.Seek(Offset: Int64; Whence: TSeekWhence=swSet): Int64;
begin
  Flush;
  Result := Stream.Seek(Offset, Whence);
end;

function TBStream.Seekable: Boolean;
begin
  Result := Stream.Seekable;
end;

function TBStream.Size: Cardinal;
begin
  Result := Stream.Size;
end;


{ TBInStream }

constructor TBInStream.Create(const Stream: TStream; BufferSize: Integer);
begin
  inherited;
  BufferRemain := 0;
end;

function TBInStream.Read(var Buf; NBytes: Cardinal): Cardinal;
var
  rbytes, remain: Cardinal;
begin
  remain := NBytes;
  while remain > 0 do
  begin
    if BufferRemain = 0 then
    begin
      BufferRemain := Stream.Read(Buffer^, BufferSize);
      if BufferRemain = 0 then
      begin
        Result := NBytes - remain;
        Exit;
      end;
      BufferCurrent := 0;
    end;
    if remain > Cardinal(BufferRemain) then
      rbytes := BufferRemain
    else
      rbytes := remain;
    Move(Buffer[BufferCurrent], PChar(@Buf)[NBytes-remain], rbytes);
    Inc(BufferCurrent, rbytes);
    Dec(BufferRemain, rbytes);
    Dec(remain, rbytes);
  end;
  Result := NBytes;
end;

function TBInStream.Write(const Buf; NBytes: Cardinal): Cardinal;
begin
  Result := 0;
end;

procedure TBInStream.Flush;
begin
  BufferRemain := 0;
end;


{ TBOutStream }

constructor TBOutStream.Create(const Stream: TStream; BufferSize: Integer);
begin
  inherited;
  BufferRemain := BufferSize;
end;

function TBOutStream.Read(var Buf; NBytes: Cardinal): Cardinal;
begin
  Result := 0
end;

function TBOutStream.Write(const Buf; NBytes: Cardinal): Cardinal;
var
  wbytes, remain: Cardinal;
begin
  remain := NBytes;
  while remain > 0 do
  begin
    if remain > Cardinal(BufferSize) then
      wbytes := BufferSize
    else
      wbytes := remain;
    if Cardinal(BufferRemain) < wbytes then
      Flush;
    Move(PChar(@Buf)[NBytes-remain], Buffer[BufferSize-BufferRemain], wbytes);
    Dec(BufferRemain, wbytes);
    Dec(remain, wbytes);
  end;
  Result := NBytes;
end;

procedure TBOutStream.Flush;
begin
  if BufferRemain = BufferSize then
    Exit;
  Stream.Write(Buffer^, BufferSize - BufferRemain);
  BufferRemain := BufferSize; 
end;

end.
